package cz.upol.jj.Rental;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Optional;

public class Rental<T extends Item & Comparable<T>> implements Iterable<RentableItem<T>> {

    private final ArrayList<RentableItem<T>> items;
    private int cash;


    public Rental() {
        items = new ArrayList<>();
        cash = 0;
    }

    public ArrayList<RentableItem<T>> getItems(){
        return items;
    }
    public void sort(){
        items.sort((RentableItem<T> a, RentableItem<T> b) -> a.compareTo(b));
    }

     public void sortByPrice(){
        items.sort((RentableItem<T> a, RentableItem<T> b) -> Integer.compare(a.getPrice(), b.getPrice()));
     }

     public RentableItem<T> add(T item, int price){
        RentableItem<T> newItem  = new RentableItem<>(item, price);
        items.add(newItem);
        return newItem;
     }

     public boolean remove(RentableItem<T> rentableItem) {
        return items.remove(rentableItem);
     }

     public RentableItem<T> find(T item, boolean rented) {
        Optional<RentableItem<T>> found = items.stream().filter(
                (RentableItem<T> a) -> a.isRented() == rented && a.getItem().equals(item)).findFirst();
        if (found.isPresent()){
            return found.get();
        } else {
            return null;
        }
     }

     public int count(T item){
        return (int) items.stream().filter((RentableItem<T> a) -> a.getItem().equals(item) && !a.isRented()).count();
     }

     public RentableItem<T> rent(T item){
        RentableItem<T> rentIt = find(item, false);
        if (rentIt == null){
            return null;
        }
        cash += rentIt.getPrice();
        rentIt.setRented(true);
        return rentIt;

     }

     public boolean retrieve(RentableItem<T> rentableItem){
        RentableItem<T> rentIt = find(rentableItem.getItem(), true);
        if (rentIt == null){
            return false;
        }
        rentIt.setRented(false);
        return true;
     }

     public int getCash() {
        return cash;
     }

     public Iterator<RentableItem<T>> iterator() {
         return items.iterator();
     }
}
